@@ -15,7 +15,7 @@ use warnings;
 use MIME::Base64;
 
 use Kernel::System::VariableCheck qw(:all);
-use Kernel::Language qw(Translatable);
+use Kernel::Language              qw(Translatable);
 
 use parent qw(Kernel::System::EventHandler);
 
@@ -78,12 +78,12 @@ sub new {
 Create a new queue element.
 
     my $Result = $MailQueue->Create(
-        ArticleID       => '...', # optional
-        MessageID       => '...', # optional (in case article id was passed this should be also)
-        Sender          => '...',
-        Recipient       => '...' || [],
-        Message         => '...',
-        Attempts        => '...', # optional
+        ArticleID => '...',         # optional
+        MessageID => '...',         # optional (in case article id was passed this should be also)
+        Sender    => '...',
+        Recipient => '...' || [],
+        Message   => '...',
+        Attempts  => '...',         # optional
     );
 
 Returns 1 or undef.
@@ -267,28 +267,28 @@ sub Create {
 Get a list of the queue elements.
 
     my $List = $MailQueue->List(
-        ID              => '...', # optional
-        ArticleID       => '...', # optional
-        Sender          => '...', # optional
-        Recipient       => '...', # optional
-        Attempts        => '...', # optional
+        ID        => '...', # optional
+        ArticleID => '...', # optional
+        Sender    => '...', # optional
+        Recipient => '...', # optional
+        Attempts  => '...', # optional
     );
 
 This returns something like:
 
     my $List = [
         {
-            ID                        => '...',
-            ArticleID                 => '...',
-            Attempts                  => '...',
-            Sender                    => '...',
-            Recipient                 => ['...'],
-            Message                   => '...',
-            DueTime                   => '...',
-            LastSMTPCode              => '...',
-            LastSMTPMessage           => '...',
+            ID              => '...',
+            ArticleID       => '...',
+            Attempts        => '...',
+            Sender          => '...',
+            Recipient       => ['...'],
+            Message         => '...',
+            DueTime         => '...',
+            LastSMTPCode    => '...',
+            LastSMTPMessage => '...',
         },
-        ...,
+        # ...
     ]
 
 =cut
@@ -369,22 +369,22 @@ sub List {
 Get a queue element.
 
     my $Item = $MailQueue->Get(
-        ID              => '...' # optional
-        ArticleID       => '...' # optional
+        ID        => '...', # optional
+        ArticleID => '...', # optional
     );
 
 This returns something like:
 
     $Item = {
-        ID                        => '...',
-        ArticleID                 => '...',
-        Attempts                  => '...',
-        Sender                    => '...',
-        Recipient                 => ['...'],
-        Message                   => '...',
-        DueTime                   => '...',
-        LastSMTPCode              => '...',
-        LastSMTPMessage           => '...',
+        ID              => '...',
+        ArticleID       => '...',
+        Attempts        => '...',
+        Sender          => '...',
+        Recipient       => ['...'],
+        Message         => '...',
+        DueTime         => '...',
+        LastSMTPCode    => '...',
+        LastSMTPMessage => '...',
     };
 
 or and empty hashref if element not found.
@@ -520,7 +520,7 @@ sub Update {
 Delete queue elements.
 
     my $Result = $MailQueue->Delete(
-        %Filters,                       # See _FiltersSQLAndBinds
+        %Filters,       # See _FiltersSQLAndBinds
     );
 
 Returns 1 or undef.
@@ -587,11 +587,11 @@ sub Delete {
 Send/Process a mail queue element/item.
 
     my $List = $MailQueue->Send(
-        ID              => '...',
-        Sender          => '...',
-        Recipient       => '...',
-        Message         => '...',
-        Force           => '...' # optional, to force the sending even if isn't time
+        ID        => '...',
+        Sender    => '...',
+        Recipient => '...',
+        Message   => '...',
+        Force     => '...',     # optional, to force the sending even if isn't time
     );
 
 This returns something like:
@@ -640,6 +640,58 @@ sub Send {
     $Param{CommunicationLogObject} = $Self->_GetCommunicationLog( %Param, );
     $Param{CommunicationID}        = $Param{CommunicationLogObject}->CommunicationIDGet();
 
+    # Increase number of attempts and delete item if limit has been reached.
+    $Param{Attempts} //= 0;
+    $Param{Attempts}++;
+    my %UpdateData = (
+        Attempts => $Param{Attempts},
+    );
+    my $ItemUpdateOK = $Self->Update(
+        Filters => {
+            ID => $Param{ID},
+        },
+        Data => \%UpdateData,
+    );
+
+    if ( !$ItemUpdateOK ) {
+        my $LogMessage = sprintf(
+            'Error while updating mail queue element "%s" with "%s"!',
+            $Param{ID},
+            join( ', ', map { $_ . '=' . $UpdateData{$_} } sort keys %UpdateData ),
+        );
+
+        $LogObject->Log(
+            Priority => 'error',
+            Message  => $LogMessage,
+        );
+
+        $Param{CommunicationLogObject}->ObjectLog(
+            ObjectLogType => 'Message',
+            Priority      => 'Error',
+            Key           => 'Kernel::System::MailQueue',
+            Value         => $LogMessage,
+        );
+
+        return;
+    }
+
+    my $Config      = $Kernel::OM->Get('Kernel::Config')->Get('MailQueue');
+    my $MaxAttempts = $Config->{ItemMaxAttempts};
+    if ( $Param{Attempts} > $MaxAttempts ) {
+
+        # This will lead to _SendError executing the code for reaching limit of attempts
+        # and hence deleting the item from the queue.
+        $Self->_SendError(
+            Item       => \%Param,
+            SendResult => {},
+        );
+
+        return {
+            Status  => 'Failed',
+            Message => 'Sending has failed.',
+        };
+    }
+
     # If DueTime is bigger than current time, skip, it is not time to run yet.
     my $CurrentSysDTObject = $Kernel::OM->Create('Kernel::System::DateTime');
     my $DueTime            = $Param{DueTime};
@@ -694,7 +746,7 @@ sub Send {
 
     return {
         Status  => 'Failed',
-        Message => 'Sending has Failed.'
+        Message => 'Sending has failed.'
     };
 }
 
@@ -708,10 +760,10 @@ triggers a Event Notification.
 
     my $Result = $Object->_SendSuccess(
         Item => {
-            ID                        => ...,
-            ArticleID                 => ..., # optional
-            UserID                    => ...,
-            CommunicationLogObject    => ...,
+            ID                     => ...,
+            ArticleID              => ...,  # optional
+            UserID                 => ...,
+            CommunicationLogObject => ...,
         }
     );
 
@@ -800,12 +852,12 @@ sub _SendSuccess {
 
 Handles Send errors.
 Situations where the mail queue item is deleted:
-    - SMTP 5?? errors codes, considered permanent errors.
-    - reached maximum attempts
+- SMTP 5?? errors codes, considered permanent errors.
+- reached maximum attempts
 
     $Object->_SendError(
-        Item       => ...,
-        SendResult => ...,
+        Item       => '...',
+        SendResult => '...',
     );
 
 This always returns undef.
@@ -829,7 +881,7 @@ sub _SendError {
     my $SendResult = $Param{SendResult};
 
     my $Item            = $Param{Item};
-    my $ItemAttempts    = $Item->{Attempts} + 1;
+    my $ItemAttempts    = $Item->{Attempts};
     my $ItemMaxAttempts = $Config->{ItemMaxAttempts};
 
     $Item->{CommunicationLogObject}->ObjectLog(
@@ -910,10 +962,10 @@ sub _SendError {
     my $NextAttempt = $CurrentSysDTObject->Clone();
     $NextAttempt->Add( Minutes => $ItemAttempts * $MinutesToIncrement );
 
-    # Update mail-queue with attempt and smtp code and message.
+    # Update mail-queue with smtp code and message.
+    # Note: Attempts already have been incremented in Send().
     my %UpdateData = (
-        Attempts => $ItemAttempts,
-        DueTime  => $NextAttempt->ToString(),
+        DueTime => $NextAttempt->ToString(),
     );
 
     if ( $SendResult->{SMTPError} ) {
@@ -969,12 +1021,12 @@ Creates or Updates the Article Transmission Error record with the error message.
 Then, fires a Notification Event.
 
     my $Result = $Object->_SetArticleTransmissionError(
-        ArticleID                 => ...,
-        Message                   => ...,
-        MessageID                 => ...,
-        UserID                    => ...,
-        ForceUpdate               => ...,
-        CommunicationLogObject    => ...,
+        ArticleID              => 123,
+        Message                => '...',
+        MessageID              => 123,
+        UserID                 => 1,
+        ForceUpdate            => '...',
+        CommunicationLogObject => '...',
     );
 
 Returns 1 or undef.
@@ -1094,17 +1146,17 @@ sub _SetArticleTransmissionError {
 Formats a Notification and asks Event Handler to send it.
 
     my $Result = $Object->_SendEventNotification(
-        ArticleID => ...,
+        ArticleID => 123,
         Status    => "Queued|Sent|Error",
-        Message   => ...,
-        UserID    => ...,
+        Message   => '...',
+        UserID    => 1,
     );
 
 This returns something like:
 
     my $Result = {
         Status  => 'Failed',
-        Message => 'Need ArticleID'
+        Message => 'Need ArticleID',
     };
 
 in case of missing or invalid arguments, or the status of the EventHandler call.
@@ -1155,12 +1207,12 @@ sub _SendEventNotification {
 Build the filter sql and associated binds.
 
     my ( $FilterSQL, $Binds ) = $MailQueue->_FiltersSQLAndBinds(
-        ID              => '...', # optional
-        ArticleID       => '...', # optional
-        CommunicationID => '...', # optional
-        Sender          => '...', # optional
-        Recipient       => '...', # optional
-        Attempts        => '...', # optional
+        ID              => 1,       # optional
+        ArticleID       => 123,     # optional
+        CommunicationID => 123,     # optional
+        Sender          => '...',   # optional
+        Recipient       => '...',   # optional
+        Attempts        => '...',   # optional
     );
 
 This returns something like:
@@ -1234,8 +1286,8 @@ sub _FiltersSQLAndBinds {
 Check if the provided email address(es) is valid.
 
     my $IsValid = $MailQueue->_CheckValidEmailAddresses(
-        ParamName => '...'       # name of the parameter that we are checking
-        Addresses => '...' || [] # email addresses to validate
+        ParamName => '...',         # name of the parameter that we are checking
+        Addresses => '...', || []   # email addresses to validate
     );
 
 Returns 1 or undef.
@@ -1300,7 +1352,9 @@ sub _CheckValidEmailAddresses {
 Check if the provided data is a non-empty hash-ref.
 
     my $IsValid = $MailQueue->_CheckValidMessageData(
-        Data => {...}
+        Data => {
+            # ...
+        },
     );
 
 Returns 1 or undef.
@@ -1589,7 +1643,7 @@ sub _SetCommunicationLogLookup {
 Get the communication log associated to the queue item, if not found, creates a new one.
 
     my $CommunicationLog = $Self->_GetCommunicationLog(
-        ID => '...' # mail-queue item ID
+        ID => '...',    # mail-queue item ID
     );
 
 Returns communication-log object.
